/****************************************************************************/
/*  C5515.cmd                                                               */
/*  Copyright (c) 2012  Texas Instruments Incorporated                      */
/*  Author: Rafael de Souza                                                 */
/*                                                                          */
/*    Description: This file is a sample linker command file that can be    */
/*                 used for linking SARAMrams built with the C compiler and  */
/*                 running the resulting .out file on a C5515.              */
/*                 Use it as a guideline.  You will want to                 */
/*                 change the memory layout to match your specific          */
/*                 target system.  You may want to change the allocation    */
/*                 scheme according to the size of your SARAMram.            */
/*                                                                          */
/****************************************************************************/

//See CSL_GPTExample from c55_lp librarires

-stack    0x3000                /* PRIMARY STACK SIZE    */
-sysstack 0x2000                /* SECONDARY STACK SIZE  */
-heap     0x2000                /* HEAP AREA SIZE        */

MEMORY
{
	//See table 2-1 of TMS320C5515 (SPRS645F)
  PAGE 0:
    MMR			  : origin = 0x000000  length = 0x0000c0  /* 192B Memory Mapped Registers */
    VEC(RWX)	  : origin = 0x0000C0  length = 0x000100	/*Beginning of DARAM*/
    DARAM(RWX)     : origin = 0x0001C0  length = 0x00FE40	/*DARAM*/
    SARAM(RWX)      : origin = 0x010000  length = 0x010000	/*SARAM*/

    CS0			  : origin = 0x050000  length = 0x7B0000  /* 8MB CS0 external memory space */
    CS2			  : origin = 0x800000  length = 0x400000  /* 4MB CS2 external memory space */	//Nor Flash
    CS3			  : origin = 0xC00000  length = 0x200000  /* 2MB CS3 external memory space */
    CS4			  : origin = 0xE00000  length = 0x100000  /* 1MB CS4 external memory space */
    CS5			  : origin = 0xF00000  length = 0x0E0000  /* 1MB CS5 external memory space */
    ROM			  : origin = 0xFE0000  length = 0x01FF00  /* 128kB ROM (MPNMC=0) or CS5 (MPNMC=1) */
    VECS		  : origin = 0xFFFF00  length = 0x000100  /* reset vector */
}

SECTIONS
{
  .intvec    : > VEC ALIGN = 256
  .text      : > SARAM
  .data      : > DARAM
  .cinit     : > SARAM
  .switch    : > SARAM
  .stack     : > SARAM
  .sysstack  : > DARAM
  .bss       : > DARAM
  .sysmem    : > SARAM
  .const     : > SARAM
  .cio	     : > SARAM
  vectors (NOLOAD)
}
