/*
 * Corr_Coeff.c
 *
 *  Created on: Oct 22, 2015
 *      Author: stsuyuki2
 *
 * Using Eq. 1 from section 2.2.3 in the Low Complexity Algorithm paper,
 * this function finds the normalized correlation coefficient of T1 or T2.
 *
 * Inputs
 * bk: signal to be correlated
 * N: template (T1 or T2)
 * N_avg: average of the template
 * L: length of the template 
 */

#include "pan_tompkins.h"

//Corr_Coeff
float32_t Corr_Coeff(int32_t* bk, int32_t* N, float32_t N_avg, int32_t L){

#ifndef ARM
//declare variables
    float32_t bk_avg, num=0, den=0, product;    
    int32_t i, temp=0;

    //determine b_k average
    for(i = 0; i < L; i++){
        temp += bk[i];
    }
    bk_avg = (float32_t) temp / (float32_t) L;

    for(i = 0; i < L; i++){
        product = ((float32_t) bk[i] - bk_avg) * ((float32_t) N[i] - N_avg);
        num += product;
        den += product*product;
    }

    den = sqrtf(den);

    return num/den;
#else
//declare variables
float32_t bk_avg, num=0, den=0, bk_den=0, N_den=0;
static float32_t temp[T2_max_length];
static float32_t temp2[T2_max_length];
static float32_t bk_norm[T2_max_length];
static float32_t N_norm[T2_max_length];
int32_t m;

//calculate bk_avg
for(m=0; m<L; m++){
		temp[m]=(float32_t)bk[m];
	}

arm_mean_f32(temp,L,&bk_avg);

//bk(n)-bk_avg
arm_fill_f32(bk_avg, temp2, L);
arm_sub_f32(temp, temp2, bk_norm, L);

//calculate N avg
for(m=0; m<L; m++){
		temp[m]=(float32_t)N[m];
	}

arm_fill_f32(N_avg, temp2, L);
//N(n)-N_avg
arm_sub_f32(temp, temp2, N_norm, L);

//sum((bk(n)-bk_avg)*(N(n)-N_avg))
arm_mult_f32(bk_norm, N_norm, temp, L);
for(m=0; m<L; m++)
{
	num+=temp[m];
}

//(bk(n)-bk_avg)^2
arm_mult_f32(bk_norm, bk_norm, temp, L);
//(N(n)-N_avg)^2
arm_mult_f32(N_norm, N_norm, temp2, L);

for(m=0; m<L; m++)
{
	bk_den+=temp[m];
	N_den+=temp2[m];
}

//sum((bk(n)-bk_avg)^2)*sum((N(n)-N_avg)^2)
bk_avg=bk_den*N_den;
den = sqrtf(bk_den*N_den);

//return normalized correlation coefficient
return(num/den);
#endif

}
