/*
 * Z_Corr.c
 *
 *  Created on: Oct 29, 2015
 *      Author: stsuyuki2
 *  This function determines the classification function, z, given by
 *  zk=(exp(xk^r)+exp(yk^r))/2e.
 *  
 *  Inputs
 *  xk: T1 correlation coefficient
 *  yk: T2 correlation coefficient
 *  r: slope (for exponentiation)
 */
#include "pan_tompkins.h"

float32_t Z_Corr(float32_t xk, float32_t yk, int32_t r)
{
	float32_t zk;
	float32_t exr = powf(xk, (float32_t)r);
	exr = expf(exr);

	float32_t eyr = powf(yk, (float32_t)r);
	eyr = expf(eyr);

	float32_t den = 2*expf(1.0f);

	zk = (exr+eyr)/den;
	return zk;
}
