/*
 * template_matching.c
 *
 *  Created on: Nov 17, 2015
 *      Author: stsuyuki2
 */

#include "pan_tompkins.h"


void generate_templates(int32_t* signal, int32_t* wavelet_5s, int32_t prev_peak_loc, int32_t curr_peak_loc, int32_t curr_interval, int32_t* T1, int32_t* T1_length, float32_t* T1_avg, int32_t* T2, int32_t* T2_length, float32_t* T2_avg, int32_t* left_marg, int32_t* right_marg, int32_t* template_found, int32_t amp_mean, int32_t interval_mean)
{
	int32_t onset=0, offset=0;

	if ((signal[prev_peak_loc]<=((110*amp_mean)/100)) && (signal[prev_peak_loc]>=((90*amp_mean)/100)) && (curr_interval>=((90*interval_mean)/100)) && (curr_interval<=((110*interval_mean)/100)))
	{
		*template_found=1;
		int32_t max_range=indexWrap(prev_peak_loc-6,FS5);
		int32_t min_range=indexWrap(prev_peak_loc+6,FS5);
		int32_t j, peak_loc=0;
		
		//find the peak location in the wavelet signal
		for (j=min_range; j!=max_range; j=indexWrap(j-1,FS5))
		{
			if(((wavelet_5s[indexWrap(j+1,FS5)]<=0) && (wavelet_5s[j]>=0)) || ((wavelet_5s[indexWrap(j+1,FS5)]>=0) && (wavelet_5s[j]<=0)))
			{
				peak_loc=j;
			}	
		}
	
		if(peak_loc!=0)
		{
			min_range=indexWrap(peak_loc+35,FS5);
			max_range=indexWrap(peak_loc+6,FS5);
			
			j=max_range;
			int32_t found_onset=0;

			//find the onset and offset in the wavelet signal
			while((j!=min_range) && (found_onset==0))
			{
				if(((wavelet_5s[indexWrap(j+1,FS5)]<=0) && (wavelet_5s[j]>=0)) || ((wavelet_5s[indexWrap(j+1,FS5)]>=0) && (wavelet_5s[j]<=0)))
				{
					onset=j;
					found_onset=1;
				}
				j=indexWrap(j+1,FS5);
			}

			min_range=indexWrap(peak_loc-6,FS5);
			max_range=indexWrap(peak_loc-35,FS5);
			
			j=min_range;
			int32_t found_offset=0;
			while((j!=max_range) && (found_offset==0))
			{
				if(((wavelet_5s[indexWrap(j+1,FS5)]<=0) && (wavelet_5s[j]>=0)) || ((wavelet_5s[indexWrap(j+1,FS5)]>=0) && (wavelet_5s[j]<=0)))
				{
					offset=j;
					found_offset=1;
				}
				j=indexWrap(j-1,FS5);
			}		
		}
		
		int32_t i=0;
		int32_t k=onset;

		//save T1 and T2
		while(k!=(indexWrap(offset-31,FS5)))
		{
			T1[i]=signal[k];
			i++;
			k=indexWrap(k-1,FS5);
		}
		
		i=0;
		k=prev_peak_loc;
		while(k!=(indexWrap(curr_peak_loc-1,FS5)))
		{
			T2[i]=signal[k];
			i++;
			k=indexWrap(k-1,FS5);
		}
		
		*left_marg = length_circ_buffer(onset,prev_peak_loc);
		*right_marg = length_circ_buffer(prev_peak_loc,indexWrap(offset-30,FS5));

		*T1_length = get_nonzero_length(T1,T1_max_length);
		*T2_length = get_nonzero_length(T2,T2_max_length);

		static float32_t float_T1[T1_max_length];
		static float32_t float_T2[T2_max_length]; //declare floating point versions of T1 and T2 for finding the average

		// determine the mean of the templates - first convert T1 to float and then perform averaging operation
		for(j=0; j<*T1_length; j++){
			float_T1[j]=(float32_t)T1[j];
		}

		for(j=0; j<*T2_length; j++){
			float_T2[j]=(float32_t)T2[j];
		}

        #ifndef ARM
   		    mean_f32(float_T1,*T1_length,T1_avg);
    		mean_f32(float_T2,*T2_length,T2_avg);
        #else
		    arm_mean_f32(float_T1,*T1_length,T1_avg);
    		arm_mean_f32(float_T2,*T2_length,T2_avg);
        #endif
	}
}

void temp_match(int32_t* signal, int32_t h_prev, int32_t h_curr, int32_t* T1, int32_t T1_length, float32_t T1_avg, int32_t* T2, int32_t T2_length, float32_t T2_avg, int32_t left_marg, int32_t right_marg, float32_t* zk_pvc, int32_t* pvc_index, int32_t* vt_i, int32_t* sendMessage, int32_t* peak_locs, int32_t peaks_index_prev)
{
	int32_t r=5;  //3.5  .56; 4.6 .55; 3' .682
	float32_t trsh=0.55; //correlation threshold (10,.615) (3,.7)
	float32_t xk, yk, zk;
	static int32_t bkx[T1_max_length];
	static int32_t bky[T2_max_length];
	static int32_t bkx_new[T1_max_length];
	static int32_t bky_new[T2_max_length];
	static int32_t j, k, bkx_nonzero_length, bky_nonzero_length;
	static int32_t total_p = 0;
	static int32_t p = 0;
	static int32_t vt_count=0;

	array_init(bkx, T1_max_length);
 	array_init(bky, T2_max_length);

 	j=indexWrap(h_prev+left_marg-1,FS5);
 	bkx_nonzero_length = left_marg+right_marg-1;

 	for(k = 0; k<bkx_nonzero_length; k++)
 	{
		bkx[k] = signal[j];
 		j=indexWrap(j-1,FS5);
	}

	j=h_prev;

	bky_nonzero_length=length_circ_buffer(h_prev,h_curr);

	for(k=0; k<bky_nonzero_length; k++)
	{
		bky[k] = signal[j];
		j=indexWrap(j-1,FS5);
	}

	if(bkx_nonzero_length != T1_length)
	{
		Stretch_Comp(bkx, T1_length, bkx_nonzero_length, bkx_new);
	}
	else
	{
		array_copy(bkx, bkx_new, bkx_nonzero_length);
	}
	if(bky_nonzero_length != T2_length)
	{
		Stretch_Comp(bky, T2_length, bky_nonzero_length, bky_new);
	}
	else
	{
		array_copy(bky, bky_new, bky_nonzero_length);
	}

	xk = Corr_Coeff(bkx_new, T1, T1_avg, T1_length);
	yk = Corr_Coeff(bky_new, T2, T2_avg, T2_length);
	zk = Z_Corr(xk, yk, r);

	if (fabs(zk) < trsh)
	{
		zk_pvc[total_p] = zk;
		pvc_index[total_p] = peak_locs[indexWrap(peaks_index_prev-1,peaks_size)]-21;
		total_p++;
		p++;
	}
	else
	{
		p = 0;
	}

	if(p>=3)
	{
		vt_i[vt_count]=peak_locs[indexWrap(peaks_index_prev-1,peaks_size)];
		vt_count++;
		*sendMessage=1;
	}
}

//determines the number of elements in a portion of a circular buffer
int32_t length_circ_buffer(int32_t prev, int32_t curr)
{
	int32_t result;
	if((prev-curr)<0)
	{
		result=(FS5-curr)+prev+1;
	}

	else
	{
		result=prev-curr+1;
	}
	return result;
}
