/*
 * ecg_interrupt.c
 *
 *  Created on: Jan 31, 2017
 *      Author: edward
 */

#include "ecg_interrupt.h"
#include "ecg.h"
#include "ecg_adc.h"
#include "ecg_buffer.h"
#include "ecg_uart.h"

Bool initialize_interrupts(void){
	/* Clear any pending interrupts */
	IRQ_clearAll();

	disable_interrupts();

	IRQ_setVecs((Uint32)(&VECSTART));

	IRQ_plug(TINT_EVENT, &gpt0ISR);
	IRQ_plug(SAR_EVENT, &sarISR);
	IRQ_plug(UART_EVENT, &uartISR);

	return TRUE;
}


Bool enable_interrupts(void){

	IRQ_enable(TINT_EVENT);
	IRQ_enable(SAR_EVENT);
	IRQ_enable(UART_EVENT);
	IRQ_globalEnable();

	return TRUE;
}
Bool disable_interrupts(void){

	/* Disable all the interrupts */
	IRQ_globalDisable();
	IRQ_disableAll();

	return TRUE;
}

interrupt void gpt0ISR(void){

	static Uint16 benchmark_data_buffer_iterator = 0;

	if(acquisition_mode == ECG_ACQUIRE_SENSOR){
		Bool sar_busy = SAR_getStatus(&hSar, &status);

		if(!sar_busy){
			status = SAR_startConversion(&hSar);
		}
		else{
			handle_error(2);
		}
	}
	else if(acquisition_mode == ECG_ACQUIRE_BENCHMARK_DATA){
		if(benchmark_data_buffer_iterator != benchmark_data_buffer_index){
			add_to_sensor_buffer(benchmark_data_buffer[benchmark_data_buffer_iterator]);
			benchmark_data_buffer_iterator = circular_buffer_index(benchmark_data_buffer_iterator,1,BENCHMARK_DATA_SIZE);
		}
	}

	/* Clear Timer Interrupt Aggregation Flag Register (TIAFR) */
	CSL_SYSCTRL_REGS->TIAFR = 0x01;
	IRQ_clear(TINT_EVENT);
}

//ADC conversion complete
interrupt void sarISR(void){
	Uint16 read_data;

	//read data from SAR buffer
	status = SAR_readData(&hSar, &read_data);
	if(CSL_SOK != status){
		handle_error(ECG_SAR_READ_FAILED);
	}
	else{
		add_to_sensor_buffer(read_data);
	}

	IRQ_clear(SAR_EVENT);
}

//UART Interrupt
interrupt void uartISR(void){
	Uint16 eventId = 0;

	IRQ_disable(UART_EVENT);

	eventId = UART_getEventId(hUart);

	if(((void(*)(void))(hUart->UART_isrDispatchTable[eventId]))){
		((void(*)(void))(hUart->UART_isrDispatchTable[eventId]))();
	}

	IRQ_enable(UART_EVENT);


	return;
}
