/*
 * ecg_uart.c
 *
 *  Created on: Feb 6, 2017
 *      Author: edward
 */

#include <csl_pll.h>
#include "ecg_uart.h"
#include "ecg.h"
#include "ecg_buffer.h"
#include "rtecg.h"

CSL_UartHandle hUart;
CSL_UartObj     uartObj;

Bool initialize_uart(){

    CSL_UartSetup	uartSetup;
    CSL_UartIsrAddr	isrAddr;
    Uint32       	uartInstId;

    uartInstId = 0;

    //Set in UART example from c55_csl_3.06.  UART does not work without, can't find documentation
    status = SYS_setEBSR(CSL_EBSR_FIELD_PPMODE, CSL_EBSR_PPMODE_1);

    status = UART_init(&uartObj, uartInstId, UART_POLLED);
    if(status != CSL_SOK){
        return FALSE;
    }

    hUart = (CSL_UartHandle) &uartObj;

    uartSetup.afeEnable = CSL_UART_NO_AFE;
    uartSetup.baud = SERIAL_BAUD;
    uartSetup.clkInput = getSysClk() * 1000;
    uartSetup.fifoControl = CSL_UART_FIFO_DMA1_DISABLE_TRIG08;
    uartSetup.loopBackEnable = CSL_UART_NO_LOOPBACK;
    uartSetup.parity = CSL_UART_DISABLE_PARITY;
    uartSetup.rtsEnable = CSL_UART_NO_RTS;
    //1 stop bit, see table 15 of TMS320C5515/14/05/04/VC05/VC04 DSP Universal Asynchronous Receiver/Transmitter User Guide
    uartSetup.stopBits = 0;
    uartSetup.wordLength = CSL_UART_WORD8;

    isrAddr.rbiAddr		= uart_rxIsr;
    isrAddr.tbeiAddr	= NULL;
    isrAddr.ctoi	 	= NULL;
    isrAddr.lsiAddr		= NULL;

    status = UART_setup(hUart, &uartSetup);
    if(status != CSL_SOK){
        return FALSE;
    }

    status = UART_setCallback(hUart, &isrAddr);
    if(status != CSL_SOK){
        return FALSE;
    }

    status = UART_eventEnable(hUart, CSL_UART_RECVOR_REG_DATA_INTERRUPT);

    return TRUE;
}

Bool parse_uart_rx(){
	static Uint16 read_buffer_iterator = 0;

	Uint16 command, argument_count;
	Int16 argument[MAX_ARGUMENT_COUNT] = {0};
	Bool command_valid = FALSE;
	int i;

	//Parse read data
	if(read_buffer_iterator != uart_read_buffer_index &&
			circular_buffer_index(read_buffer_iterator,1,UART_READ_BUFFER_SIZE) != uart_read_buffer_index &&
			circular_buffer_index(read_buffer_iterator,2,UART_READ_BUFFER_SIZE) != uart_read_buffer_index &&
			circular_buffer_index(read_buffer_iterator,3,UART_READ_BUFFER_SIZE) != uart_read_buffer_index){

		command_valid = TRUE;

		if(UART_read_buffer[read_buffer_iterator] == PACKET_CONST_H &&
				UART_read_buffer[circular_buffer_index(read_buffer_iterator,1,UART_READ_BUFFER_SIZE)] == PACKET_CONST_L){

			command = UART_read_buffer[circular_buffer_index(read_buffer_iterator,2,UART_READ_BUFFER_SIZE)];
			argument_count = UART_read_buffer[circular_buffer_index(read_buffer_iterator,3,UART_READ_BUFFER_SIZE)];

			for(i = 0; i < argument_count; i++){
				if(circular_buffer_index(read_buffer_iterator, 4 + 2*i ,UART_READ_BUFFER_SIZE) != uart_read_buffer_index &&
						circular_buffer_index(read_buffer_iterator, 5 + 2*i ,UART_READ_BUFFER_SIZE) != uart_read_buffer_index){
					argument[i] = (UART_read_buffer[circular_buffer_index(read_buffer_iterator, 4 + 2*i ,UART_READ_BUFFER_SIZE)] << 8)|
									UART_read_buffer[circular_buffer_index(read_buffer_iterator, 5 + 2*i ,UART_READ_BUFFER_SIZE)];
				}
				else{
					command_valid = FALSE;
					break;
				}
			}
			if(command_valid){
				read_buffer_iterator = circular_buffer_index(read_buffer_iterator, 4 + 2*argument_count ,UART_READ_BUFFER_SIZE);
			}
		}
		else{
			read_buffer_iterator = circular_buffer_index(read_buffer_iterator,1,UART_READ_BUFFER_SIZE);
		}
	}

	//Handle command
		if(command_valid){
			switch(command){
			case CMD_STOP:
				acquisition_mode = ECG_ACQUIRE_NONE;
				break;
			case CMD_ACQUIRE_ECG:
				acquisition_mode = ECG_ACQUIRE_SENSOR;
				break;
			case CMD_ACQUIRE_BENCHMARK_DATA:
				acquisition_mode = ECG_ACQUIRE_BENCHMARK_DATA;
				break;
			case CMD_BENCHMARK_DATA:
				for(i = 0; i < argument_count; i++){
					benchmark_data_buffer[benchmark_data_buffer_index] = argument[i];
					benchmark_data_buffer_index = circular_buffer_index(benchmark_data_buffer_index,1,BENCHMARK_DATA_SIZE);
				}
				break;
			default:
				break;
			}
		}
	return command_valid;
}


void uart_rxIsr(void){
	char read_buffer[8] = {0};
	int i = 0;
    UART_read(hUart, read_buffer, 8, UART_READ_TIMEOUT);
    for(i = 0; i < 8; i++){
    	UART_read_buffer[uart_read_buffer_index] = read_buffer[i];
    	uart_read_buffer_index = circular_buffer_index(uart_read_buffer_index, 1, UART_READ_BUFFER_SIZE);
    }
	return;
}

Bool write_integer_uart(Int32 value){
    char c[12] = "\n";
    Uint16 digits = 0;
    Uint16 negative = value < 0;
    if(negative){
        value *= -1;
    }

    do{
        digits++;
        c[digits] = '0' + (value % 10);
        value /= 10;
    }while(value != 0);

    if(negative){
        digits++;
        c[digits] = '-';
    }

    int i;
    for(i=digits; i >= 0; i--){
        status = UART_write(hUart, &c[i], 1, UART_WRITE_TIMEOUT);
        if(status != CSL_SOK){
            return FALSE;
        }
    }

    return TRUE;
}



int write_uart(Uint8 value) {
    int ret;

    ret = UART_write(hUart, (char*)&value, 1, UART_WRITE_TIMEOUT);
    if (ret != CSL_SOK) {
        return 1;
    }

    return 0;
}



int write_uart16(Uint16 value) {
    int ret;
    Uint8 byte_hi;
    Uint8 byte_lo;

    byte_hi = ((value >> 8) & 0xFF);
    byte_lo = (value & 0xFF);

    ret = UART_write(hUart, (char*)&byte_hi, 1, UART_WRITE_TIMEOUT);
    if (ret != CSL_SOK) {
        return 1;
    }

    ret = UART_write(hUart, (char*)&byte_lo, 1, UART_WRITE_TIMEOUT);
    if (ret != CSL_SOK) {
        return 1;
    }

    return 0;

}


/* Specialized function to write a flag to the serial port, according to
 * the communication protocol between the Pi and DSP.
 */
int write_flag(Uint8 flag) {
    write_uart16(MAKE_FLAG(flag));
    write_uart16(MAKE_FLAG(flag));
    write_uart16(MAKE_FLAG(flag));

    return 0;
}
