/*
 * main.c
 */

#include "ecg.h"
#include "ecg_adc.h"
#include "ecg_buffer.h"
#include "ecg_gpio.h"
#include "ecg_interrupt.h"
#include "ecg_timer.h"
#include "ecg_uart.h"

#include "pan_tompkins.h"

#include "rtecg.h"

Uint8 select_buffer_flag = BUFFER_FILT;

PLL_Obj pllObj;
PLL_Config pllCfg1;
PLL_Handle hPll;
PLL_Config pllCfg_12p288MHz = {0x8173, 0x8000, 0x0806, 0x0000};
PLL_Config pllCfg_40MHz     = {0x8988, 0x8000, 0x0806, 0x0201};
PLL_Config pllCfg_60MHz     = {0x8724, 0x8000, 0x0806, 0x0000};
PLL_Config pllCfg_75MHz     = {0x88ED, 0x8000, 0x0806, 0x0000};
PLL_Config pllCfg_100MHz    = {0x8BE8, 0x8000, 0x0806, 0x0000};
PLL_Config pllCfg_120MHz    = {0x8E4A, 0x8000, 0x0806, 0x0000};
PLL_Config *pPLLConfigInfo;

int main(void){

	//setup PLL
    status = PLL_init(&pllObj, CSL_PLL_INST_0);
    if(CSL_SOK != status)
    {
       handle_error(ECG_ERROR);
    }
	hPll = (PLL_Handle)(&pllObj);
	PLL_reset(hPll);
	pPLLConfigInfo = &pllCfg_120MHz;
	status = PLL_config (hPll, pPLLConfigInfo);
	if(CSL_SOK != status)
	{
	   handle_error(ECG_ERROR);
	}

	//acquisition_mode = ECG_ACQUIRE_BENCHMARK_DATA;

    Uint16 i;
    Uint16 ret;
    Uint16 write_buffer_index;
    Int32 bpf_buffer[SENSOR_BUFFER]; /* Buffer of band pass filtered data */
    Int32 write_buffer[SENSOR_BUFFER]; /* Buffer that is transmitted to logger */
    Int32 qrs_buffer[peaks_size]; /* Buffer that is transmitted to logger */
    Int32 pvc_buffer[peaks_size]; /* Buffer that is transmitted to logger */
    Int32 arrhythmia_detected = 0; /* Algorithm detection result */


    ret = initialize_interrupts();
    handle_error(ret);

    ret = initialize_gpio();
    handle_error(ret);

    ret = initialize_adc();
    handle_error(ret);

    ret = initialize_timers();
    handle_error(ret);

    ret = initialize_buffers();
    handle_error(ret);

    ret = initialize_uart();
    handle_error(ret);

    ecg_sensor_buffer_index = 0; /* Used in ADC ISR */
    write_buffer_index = 0;
    input_buffer_ready = FALSE; /* Start with empty input buffer */
    enable_interrupts();

    /////////////////////////////////////////////////////////////////////////
    //Main Loop
    while (1) {
        //display_int_led(ecg_sensor_buffer[index] >> 6);

        /* If data has accumulated in the input buffer... */
        if (input_buffer_ready) {
            /* Run the arrhythmia detection algorithm, with the blue LED on
             * during its execution. */

            /* If we want the raw input buffer to be transmitted, we have to 
               copy it to write_buffer now, since arrhythmia_detection() 
               destroys it. */
            if (select_buffer_flag == BUFFER_RAW) {
                memcpy(write_buffer, ecg_input_buffer, sizeof(write_buffer));
            }

            display_int_led(0);
            arrhythmia_detection(ecg_input_buffer, &arrhythmia_detected, bpf_buffer,
                                 qrs_buffer, pvc_buffer);
            display_int_led(1);

            /* Otherwise, after the algorithm is run, copy the result into
               write_buffer */
            if (select_buffer_flag == BUFFER_FILT) {
                memcpy(write_buffer, bpf_buffer, sizeof(write_buffer));
            }

            /* Now transmit QRS analysis data and data buffer */
            write_flag(FLAG_START);

            /* Transmit the QRS data buffers*/
            if (arrhythmia_detected != 0) {
                write_flag(FLAG_ARRHYTHMIA_DETECTED);
            }

            write_flag(FLAG_SEND_QRS);

            for(i = 0; i < peaks_size; i++){
                write_uart16(qrs_buffer[i]);
            }

            write_flag(FLAG_END_BUFFER);

            /* Transmit the PVC data buffers*/
            write_flag(FLAG_SEND_PVC);

            for(i = 0; i < peaks_size; i++){
                write_uart16(pvc_buffer[i]);
            }

            write_flag(FLAG_END_BUFFER);

            
            /* Transmit the ECG data buffers */
            if (select_buffer_flag == BUFFER_FILT) {
                write_flag(FLAG_SEND_FILT);
            } else if (select_buffer_flag == BUFFER_RAW) {
                write_flag(FLAG_SEND_RAW);
            }

            do {
                write_uart16(write_buffer[write_buffer_index]);
                write_buffer_index = circular_buffer_index(write_buffer_index, 1, SENSOR_BUFFER);
            } while (write_buffer_index != 0);

            write_flag(FLAG_END_BUFFER);


            /* Done transmitting stuff, wrap up */
            write_flag(FLAG_STOP);

            /* Once all that is done, we declare the input buffer empty again. */
            input_buffer_ready = FALSE;
            /*write_buffer_empty = FALSE;*/

        }

        parse_uart_rx();

    }//end main loop
}


